==> openobd_utils-0.13.0.tar.gz: openobd_utils-0.13.0/src/openobd_utils/isotp_socket.py <==
import time

from openobd import *
from .stream_manager import StreamManager, TimeoutException
from .response_exceptions import *


class IsotpSocket(StreamManager):

    def __init__(self, client: OpenOBDSession, isotp_channel: IsotpChannel, timeout: float | None = 10):
        """
        Handles sending messages to the given channel and parsing incoming responses.

        :param client: an authenticated OpenOBDClient with which to start the stream.
        :param isotp_channel: the channel which to send messages to.
        :param timeout: time in seconds to wait for each response before raising a NoResponseException. None will wait forever.
        """
        super().__init__(client.open_channel)
        self.channel = isotp_channel
        self.timeout = timeout

    def request(self, payload: str, timeout: float | None = None, silent: bool = False, tries: int = 1) -> str | None:
        """
        Sends the given payload and waits for the response. Raises the appropriate ResponseException when receiving a
        negative response.

        :param payload: the bytes to send to the vehicle.
        :param timeout: time in seconds to wait for a response before raising a NoResponseException. Will use self.timeout if None.
        :param silent: if True, will not raise any ResponseExceptions. Instead, returns the payload of negative responses or None for NoResponseExceptions.
        :param tries: how many times to send the payload until a positive response is received.
        :return: the payload of the received response.
        """
        assert tries > 0, "Optional argument 'tries' must be greater than zero."

        message = IsotpMessage(channel=self.channel, payload=payload)
        timeout = timeout if timeout is not None else self.timeout

        for current_try in range(1, tries + 1):
            self.send(message, clear_received_messages=True)

            try:
                end_time = time.time() + timeout if timeout is not None else None
                try:
                    return self._receive_and_handle_response(payload, timeout)
                except BusyRepeatRequestException as e:
                    # We should retry the request after waiting a little
                    if end_time is not None and time.time() + 0.5 > end_time:
                        # We don't have enough time left this try, so raise the received exception
                        raise e
                    time.sleep(0.5)
                    try:
                        # Make another request in the time we have left
                        remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                        return self.request(payload, timeout=remaining_time)
                    except NoResponseException:
                        # We did not receive another response in time, so raise the exception received earlier
                        raise e

            except ResponseException as e:
                if current_try < tries:
                    continue
                if silent:
                    return e.get_response()
                raise e

    def _receive_and_handle_response(self, request_payload: str, timeout: float | None) -> str:
        end_time = time.time() + timeout if timeout is not None else None

        try:
            response = self.receive(timeout=timeout)    # type: IsotpMessage
        except TimeoutException:
            raise NoResponseException(request=request_payload, response=None, request_id=self.channel.request_id, response_id=self.channel.response_id)

        response_payload = response.payload.upper()

        try:
            self._check_response_format(response_payload)

            if not self._response_matches_request(response_payload, request_payload):
                # The received message is not a response to our request, so continue listening in the time we have left
                remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                return self._receive_and_handle_response(request_payload, remaining_time)

            try:
                self._check_response_payload(response_payload)
            except RequestCorrectlyReceivedResponsePendingException as e:
                try:
                    # We need to wait a little longer for the response, so continue listening in the time we have left
                    remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                    return self._receive_and_handle_response(request_payload, remaining_time)
                except NoResponseException:
                    # Did not receive a timely response, so raise the original ResponsePending exception
                    raise e

        except ResponseException as e:
            # Add the request and response metadata to the exception before raising it
            e.set_request(request_payload)
            e.set_response(response_payload, overwrite=False)
            e.set_request_id(self.channel.request_id)
            e.set_response_id(self.channel.response_id)
            raise e

        return response_payload

    @staticmethod
    def _check_response_format(response: str) -> None:
        """
        Checks whether the response is formatted correctly. If not, raises an InvalidResponseException.

        :param response: the payload of the response.
        """
        if response[:2] == "7F" and len(response) != 6:
            raise InvalidResponseException()

    @staticmethod
    def _check_response_payload(payload: str) -> None:
        """
        Checks whether the given response is a negative response. If so, raises the appropriate exception.

        :param payload: the payload of the response.
        """
        if payload[:2] != "7F":
            return
        negative_response_code = payload[4:6]
        exception = UnknownNegativeResponseException
        if negative_response_code in negative_response_code_exceptions:
            exception = negative_response_code_exceptions[negative_response_code]
        raise exception

    @staticmethod
    def _response_matches_request(response: str, request: str) -> bool:
        """
        Checks whether the service of the response matches the service of the request.

        :param response: the payload of the response.
        :param request: the payload of the request.
        :return: True if the response matches the request. False if there's a mismatch.
        """
        service_byte = int(request[:2], 16)
        if response[:2] == "7F":
            if int(response[2:4], 16) != service_byte:
                return False
        elif int(response[:2], 16) != (service_byte + 0x40):
            return False
        return True

==> openobd_utils-1.3.0.tar.gz: openobd_utils-1.3.0/src/openobd_utils/isotp_socket.py <==
import time

from openobd import *
from .stream_manager import StreamManager, StreamTimeoutException
from .response_exceptions import *


class IsotpSocket(StreamManager):

    def __init__(self, openobd_session: OpenOBDSession, isotp_channel: IsotpChannel, timeout: float | None = 10):
        """
        Handles sending messages to the given channel and parsing incoming responses.

        :param openobd_session: an active OpenOBDSession with which to start the CAN stream.
        :param isotp_channel: the channel which to send messages to.
        :param timeout: time in seconds to wait for each response before raising a NoResponseException. None will wait forever.
        """
        super().__init__(openobd_session.open_isotp_stream)
        self.channel = isotp_channel
        self.timeout = timeout

    def request(self, payload: str, timeout: float | None = None, silent: bool = False, tries: int = 1) -> str | None:
        """
        Sends the given payload and waits for the response. Raises the appropriate ResponseException when receiving a
        negative response.

        :param payload: the bytes to send to the vehicle.
        :param timeout: time in seconds to wait for a response before raising a NoResponseException. Will use self.timeout if None.
        :param silent: if True, will not raise any ResponseExceptions. Instead, returns the payload of negative responses or None for NoResponseExceptions.
        :param tries: how many times to send the payload until a positive response is received.
        :return: the payload of the received response.
        """
        assert tries > 0, "Optional argument 'tries' must be greater than zero."

        message = IsotpMessage(channel=self.channel, payload=payload)
        timeout = timeout if timeout is not None else self.timeout

        for current_try in range(1, tries + 1):
            self.send(message, clear_received_messages=True)

            try:
                end_time = time.time() + timeout if timeout is not None else None
                try:
                    return self._receive_and_handle_response(payload, timeout)
                except BusyRepeatRequestException as e:
                    # We should retry the request after waiting a little
                    if end_time is not None and time.time() + 0.5 > end_time:
                        # We don't have enough time left this try, so raise the received exception
                        raise e
                    time.sleep(0.5)
                    try:
                        # Make another request in the time we have left
                        remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                        return self.request(payload, timeout=remaining_time)
                    except NoResponseException:
                        # We did not receive another response in time, so raise the exception received earlier
                        raise e

            except ResponseException as e:
                if current_try < tries:
                    continue
                if silent:
                    return e.get_response()
                raise e

    def _receive_and_handle_response(self, request_payload: str, timeout: float | None) -> str:
        end_time = time.time() + timeout if timeout is not None else None

        try:
            response = self.receive(timeout=timeout)    # type: IsotpMessage
        except StreamTimeoutException:
            raise NoResponseException(request=request_payload, response=None, request_id=self.channel.request_id, response_id=self.channel.response_id)

        response_payload = response.payload.upper()

        try:
            self._check_response_format(response_payload)

            if not self._response_matches_request(response_payload, request_payload):
                # The received message is not a response to our request, so continue listening in the time we have left
                remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                return self._receive_and_handle_response(request_payload, remaining_time)

            try:
                self._check_response_payload(response_payload)
            except RequestCorrectlyReceivedResponsePendingException as e:
                try:
                    # We need to wait a little longer for the response, so continue listening in the time we have left
                    remaining_time = max(0.0, end_time - time.time()) if end_time is not None else None
                    return self._receive_and_handle_response(request_payload, remaining_time)
                except NoResponseException:
                    # Did not receive a timely response, so raise the original ResponsePending exception
                    raise e

        except ResponseException as e:
            # Add the request and response metadata to the exception before raising it
            e.set_request(request_payload)
            e.set_response(response_payload, overwrite=False)
            e.set_request_id(self.channel.request_id)
            e.set_response_id(self.channel.response_id)
            raise e

        return response_payload

    @staticmethod
    def _check_response_format(response: str) -> None:
        """
        Checks whether the response is formatted correctly. If not, raises an InvalidResponseException.

        :param response: the payload of the response.
        """
        if response[:2] == "7F" and len(response) != 6:
            raise InvalidResponseException()

    @staticmethod
    def _check_response_payload(payload: str) -> None:
        """
        Checks whether the given response is a negative response. If so, raises the appropriate exception.

        :param payload: the payload of the response.
        """
        if payload[:2] != "7F":
            return
        negative_response_code = payload[4:6]
        exception = UnknownNegativeResponseException
        if negative_response_code in negative_response_code_exceptions:
            exception = negative_response_code_exceptions[negative_response_code]
        raise exception

    @staticmethod
    def _response_matches_request(response: str, request: str) -> bool:
        """
        Checks whether the service of the response matches the service of the request.

        :param response: the payload of the response.
        :param request: the payload of the request.
        :return: True if the response matches the request. False if there's a mismatch.
        """
        service_byte = int(request[:2], 16)
        if response[:2] == "7F":
            if int(response[2:4], 16) != service_byte:
                return False
        elif int(response[:2], 16) != (service_byte + 0x40):
            return False
        return True
